<?php
/**
 * [Controller]item/InputController：商品入力画面表示・登録処理コントローラ
 * 商品情報入力画面の表示および登録処理（Insert/Update）を行う
 */
class InputController extends BaseController{
	protected $param = []; //viewに渡すパラメータ
	protected $title = '商品編集';

	function get($ret = []){
		//トークンをいったんリセット
		Session::destroy("fixToken");

		//編集モードはデータを取得する
		if(Helper::ak("action",$ret["route"]) && $ret["route"]["action"] == "Edit" && Helper::ak("target",$ret["route"])){
			$modl = DbModel::read("Item");
			$this->param["iData"] = $modl->getInfoByID($ret["route"]["target"]);
		}

		$this->param['title'] = $this->title;
		return $this->param;
	}

	function post($ret = []){
		$modl = DbModel::read("Item");

		//編集モードはデータを取得する
		if(Helper::ak("action",$ret["route"]) && $ret["route"]["action"] == "Edit" && Helper::ak("target",$ret["route"])){
			$this->param["iData"] = $modl->getInfoByID($ret["route"]["target"]);
		}

		switch($ret["route"]["action"]){
			case "Add":
				$err = Validation::validate("ItemInput");
				break;
			case "Edit":
				$err = Validation::validate("ItemEdit");
				break;
		}

		$tkerr = Validation::single_validate(Input::get("tk"),"token");

		if(count($tkerr) > 0){
			$err["common"] = $tkerr;
		}

		if(count($err) > 0){
			$this->param['error'] = $err;
			//入力エラーをログに保存
			$lor = [];
			foreach($err as $k=>$v){
				$lor[] = $k."->".join("/",$v);
			}
			Log::set("Input error:\n".join("\n",$lor));

		}else{

			try{
				//トランザクション開始
				DbModel::Tran();

				//エラーなければ更新処理
				$res = false;
				switch($ret["route"]["action"]){
					case "Add":
						$res = $modl->insert("ItemInput");
						break;
					case "Edit":
						$where = [
							"item_id"=>$ret["route"]["target"]
						];
						$res = $modl->update("ItemInput",$where);

						break;
				}
				if(!$res){
					//更新失敗
					DbModel::RollBack();//ロールバック
					Log::set("Update Error");
					$this->param['error'] = ["common"=>[Config::get("message","updateError")]];

				}else{
					DbModel::Commit();//コミット

					Log::set("Item {$ret["route"]["action"]} target:{$ret["route"]["target"]}");
					Helper::redirect("ItemInput/Complete/{$ret["route"]["target"]}");
					return;
				}

			}catch (Exception $e){
				DbModel::RollBack();//ロールバック
				Log::set("DB Error:".$e->getMessage());
			}
		}
		$this->param['title'] = $this->title;
		return $this->param;
	}


}