<?php
/**
 * [Core]DbModel：DB処理コントローラ
 * DB接続に関係する処理を行う
 *
 * @link http://komaphp.k-akimoto-matsu.net/Reference#subtitle0 リファレンス@Core.DbModel
 * @author TakanoriMatsushige < http://k-akimoto-matsu.net/Inquiry >
 * @copyright k_akimoto_matsu. All Rights Reserved
 */
 class DbModel{
	/**
	 * Modelを読み込む
	 * @param unknown $className
	 * @return NULL|unknown
	 */
	public static function read($className = null){
		if(is_null($className) || $className == "") return null;
		//Model
		require_once __DIR__.DS.'..'.DS.'Model'.DS.'Model.php';

		$classFile = __DIR__.DS.'..'.DS.'Model'.DS.$className.'.php';
		if(!file_exists($classFile)){
			return null;
		}else{
			require_once $classFile;
			return new $className;
		}
	}

	/**
	 * DB接続を行う
	 * @throws Exception
	 * @return NULL|unknown|boolean|NULL|resource
	 */
	public static function connect(){

		try {
			if(!is_null(Session::get("DBCon"))){
				return Session::get("DBCon");
			}
			$db = Config::get("database","db");
			$dbset = Config::get("database",$db);
			$c = null;
			switch($db){
				case "mysql":
					$c = mysqli_connect($dbset["host"],$dbset["user"],$dbset["pass"],$dbset["dbname"]);
					mysqli_set_charset($c,Config::get("database","charset"));
					break;
				case "postgres":
					$c = @pg_connect("host={$dbset["host"]} dbname={$dbset["dbname"]} user={$dbset["user"]} password={$dbset["pass"]}");
					break;
			}

			if(!$c){
				Log::set("$db {$dbset["host"]} {$dbset["user"]} {$dbset["pass"]} {$dbset["dbname"]}");
				throw  new Exception("DB connect Error");
			}
			Session::set("DBCon",$c);
			return $c;

		}catch (Exception $e){
			Log::set("DB Connect Exception msg:".$e->getMessage());
			return false;
		}
	}

	/**
	 * 直近のDBエラーを取得する
	 * @param unknown $con
	 * @return unknown|string
	 */
	public static function getLastError($con){
		$db = Config::get("database","db");
		switch($db){
			case "mysql":
				return $con->error;
				break;
			case "postgres":
				return  pg_last_error($con);
				break;
		}
	}

	/**
	 * SQL実行
	 * 即実行のためユーザーからの入力を受け付ける場合は
	 * DbModel::es()でエスケープしてから渡すこと！！
	 * @param unknown $sql
	 * @return NULL|resource
	 */
	public static function execute($sql = null){

		if(is_null($sql) || $sql == "") return null;
		return self::query($sql);

	}
	/**
	 * Queryを実行する
	 * @param unknown $sql
	 * @return NULL|resource|boolean
	 */
	public static function query($sql = null){
		try{
			$con = self::connect();
			$db = Config::get("database","db");
			if(is_null($db) || !$con){
				throw  new Exception("DB connect Error");
			}
			switch($db){
				case "mysql":
					$res =  mysqli_query($con,$sql);
					break;
				case "postgres":
					$res =  pg_query($con,$sql);
					break;
			}
			if(!$res){
				Log::set("error!!:".self::getLastError($con)."\nSQL:".$sql);
			}
			return $res;
		}catch(Exception $e){
			Log::set("SQL Exception SQL:{$sql} msg:".$e->getMessage());
			die("データベースコネクションが確立できません。アクセス情報を確認してください。");
		}
	}

	/**
	 * トランザクションを実行する
	 * @return NULL|resource
	 */
	public static function Tran(){
		return self::execute("BEGIN");
	}

	/**
	 * ロールバックを行う
	 * @return NULL|resource
	 */
	public static function RollBack(){
		return self::execute("ROLLBACK");
	}

	/**
	 * コミットを実行する
	 * @return NULL|resource
	 */
	public static function Commit(){
		return self::execute("COMMIT");
	}

	/**
	 * SELCT実行
	 * @param unknown $table 対象テーブル
	 * @param array $where key=>value でwhere条件
	 * @param string $one 最初の1件だけとる場合
	 * @param array $order 表示順　key=>value value:0=asc 1=desc
	 * @return NULL
	 */
	public static function select($table=null,$where=[],$one = false,$order = []){

		if(is_null($table) || $table == "") return null;

		$w = null;
		if(count($where) > 0){
			$w = " where ";
			$cnt = 0;
			foreach($where as $k=>$v){
				if($cnt > 0){
					$w .=" and ";
				}
				if(is_null($v)){
					$w .= " {$k} is null ";
				}else{
					$w .= " {$k}=".self::es($v)." ";
				}
				$cnt++;
			}
		}
		$o = null;
		if(count($order) > 0){
			$o = " order by  ";
			$cnt = 0;
			foreach($order as $k=>$v){
				if($cnt > 0){
					$o .=" , ";
				}
				$by = null;
				switch($v){
					case 0:
						$by = " asc ";
						break;
					case 1:
						$by = " desc ";
						break;

				}
				$o .= " {$k} {$by} ";
				$cnt++;
			}
		}
		$sql = "select * from {$table} {$w} {$o}";

		Log::set($sql);
		$ret = self::execute($sql);

		$datas = [];
		$cnt=0;

		while($row = self::fetch($ret)){
			$datas[$cnt] = $row;
			$cnt++;
		}
		if(count($datas) > 0 && $one){
			return $datas[0];
		}
		return $datas;
	}

	/**
	 * SQL実行結果をFetchする
	 * @param unknown $ret
	 * @return unknown
	 */
	public static function fetch($ret){
		$db = Config::get("database","db");
		switch($db){
			case "mysql":
				return mysqli_fetch_assoc($ret);
				break;
			case "postgres":
				return pg_fetch_assoc($ret);
				break;
		}

	}

	/**
	 * データ挿入
	 * @param unknown $table
	 * @param unknown $data
	 * @return NULL
	 */
	public static function insert($table,$data){

		if(is_null($table) || $table == "" || is_null($data) || count($data) == 0) return null;

		$cell = null;
		$val = null;
		$cnt = 0;
		foreach($data as $k=>$v){
			if($cnt > 0){
				$cell .=" , ";
				$val .=" , ";
			}
			$cell .= self::es($k,false)." ";
			if(!is_null($v)){
				$val .= self::es($v)." ";
			}else{
				$val .= "NULL ";
			}

			$cnt++;
		}

		$sql = "insert into {$table} ({$cell}) values ({$val})";
		Log::set($sql);
		return self::execute($sql);
	}

	/**
	 * データ更新
	 * @param unknown $table
	 * @param unknown $data
	 * @return NULL
	 */
	public static function update($table,$data,$where=[]){

		if(is_null($table) || $table == "" || is_null($data) || count($data) == 0) return null;

		$cell = null;
		$val = null;
		$cnt = 0;
		$cols = [];
		foreach($data as $k=>$v){
			$cell = self::es($k,false)." ";
			if(!is_null($v)){
				$val = self::es($v)." ";
			}else{
				$val = "NULL";
			}
			$cols[] = " ".$cell."=".$val." ";
			$cnt++;
		}
		$w = null;
		if(count($where) > 0){
			$w = " where ";
			$cnt = 0;
			foreach($where as $k=>$v){
				if($cnt > 0){
					$w .=" and ";
				}
				if(is_null($v)){
					$w .= " {$k} is null ";
				}else{
					$w .= " {$k}=".self::es($v)." ";
				}
				$cnt++;
			}
		}

		$sql = "update {$table} set ".join(" , ",$cols)." ".$w;
		$res = self::execute($sql);

		//影響行数を返却
		$db = Config::get("database","db");
		switch($db){
			case "mysql":
				return $res;
				break;
			case "postgres":
				return pg_affected_rows($res);
				break;
		}
	}

	/**
	 * DELETE実行
	 * @param unknown $table 対象テーブル
	 * @param array $where key=>value でwhere条件
	 * @return NULL
	 */
	public static function delete($table=null,$where=[]){

		if(is_null($table) || $table == "") return null;

		$w = null;
		if(count($where) > 0){
			$w = " where ";
			$cnt = 0;
			foreach($where as $k=>$v){
				if($cnt > 0){
					$w .=" and ";
				}
				if(is_null($v)){
					$w .= " {$k} is null ";
				}else{
					$w .= " {$k}=".self::es($v)." ";
				}
				$cnt++;
			}
		}

		$sql = "delete from {$table} {$w}";

		$res = self::execute($sql);

		//影響行数を返却
		$db = Config::get("database","db");
		switch($db){
			case "mysql":
				return $res;
				break;
			case "postgres":
				return pg_affected_rows($res);
				break;
		}
	}

	/**
	 * self::escape()のエイリアス
	 * @param string $value
	 * @param string $vFlg
	 * @return string
	 */
	public static function es($value="",$vFlg=true){
		return self::escape($value,$vFlg);
	}

	/**
	 * SQLエスケープを行う
	 * @param unknown $value
	 * @param unknown $vFlg
	 * @return string|unknown
	 */
	private function escape($value,$vFlg){
		$db = Config::get("database","db");
		switch($db){
			case "mysql":
				$con = self::connect();
				if($vFlg){
					return "'".mysqli_real_escape_string($con,$value)."'";
				}else{
					return mysqli_real_escape_string($con,$value);
				}
				break;
			case "postgres":
				if($vFlg){
					return "'".pg_escape_string($value)."'";
				}else{
					return pg_escape_string($value);
				}
				break;
		}
	}
}