<?php
/**
 * [Core]Helper：ヘルパーコントローラ
 * その他便利機能
 *
 * @link http://komaphp.k-akimoto-matsu.net/Reference#subtitle0 リファレンス@Core.Helper
 * @author TakanoriMatsushige < http://k-akimoto-matsu.net/Inquiry >
 * @copyright k_akimoto_matsu. All Rights Reserved
 */
class Helper{

	/**
	 * サイト内指定ページにリダイレクト
	 * $toはシステムルート配下から
	 * 入ってなければトップページ
	 * @param unknown $to
	 */
	public static function redirect($to=null){
		header("Location:".Config::get("app","httproot").$to);
		exit;
	}

	/**
	 * array_key_existsのエイリアス
	 * @param unknown $key
	 * @param unknown $array
	 */
	public static function ak($key,$array){
		if(!is_array($array)) return false;
		if(is_null($key)) return false;
		return array_key_exists($key, $array);
	}

	/**
	 * urlencodeのスラッシュ対応
	 * @param unknown $value
	 * @return boolean|unknown
	 */
	public static function urlen($value= null){
		if(is_null($value)) return false;
		return urlencode(str_replace("/",Config::get("app","SLASH"),$value));
	}

	/**
	 * urldecodeのスラッシュ対応
	 * @param unknown $value
	 * @return boolean|unknown
	 */
	public static function urlde($value= null){
		if(is_null($value)) return false;
		return urldecode(str_replace(Config::get("app","SLASH"),"/",$value));
	}

	/**
	 * ランダムな半角英数文字列を作成
	 * @param number $cnt
	 * @return NULL|string
	 */
	public static function createMultiString($cnt = 5){
		$str = null;

		$pstring = "abcdefghijklmnopqrstuvdxyzABCDEFGHIJKLMNOPQRSTUVDXYZ0123456789";
		for($i=0;$i<$cnt;$i++){
			$str .= substr($pstring,rand(0,61),1);
		}

		return $str;
	}

	/**
	 * 指定ディレクトリ配下のファイル名一覧を取得する
	 * @param unknown $dir
	 * @param array $filenames
	 * @return string
	 */
	public static function getFilenameInDir($dir=null,&$filenames=[]){
		if(is_null($dir)) return [];
		if($handle = opendir($dir)){
			while(false !== ($item = readdir($handle))){
				if($item != "." && $item != ".."){
					if(is_dir($dir.DS.$item)){
						self::getFilenameInDir($dir.DS.$item,$filenames);
					}else{
						$filenames[] =$dir.DS.$item;
					}
				}
			}
		}
		closedir($handle);
		return $filenames;
	}

	/**
	 * 指定ディレクトリ配下のファイルを（ディレクトリ含めて）削除する
	 * @param unknown $dir
	 */
	public static function remove_dir($dir=null){
		if(is_null($dir)) return false;
		if(!file_exists($dir)) return false;

		if($handle = opendir($dir)){
			while(false !== ($item = readdir($handle))){
				if($item != "." && $item != ".."){
					if(is_dir($dir.DS.$item)){
						self::remove_dir($dir.DS.$item);
					}else{
						unlink($dir.DS.$item);
					}
				}
			}
		}
		closedir($handle);
		rmdir($dir);
	}

	/**
	 * JSONエンコード（エンコーディングも込み）
	 * @param array $data
	 * @return string
	 */
	public static function jsonen($data = []){
		return mb_convert_encoding(json_encode($data, JSON_UNESCAPED_UNICODE|JSON_UNESCAPED_SLASHES ),"UTF8","UTF8");
	}

	/**
	 * バイトを大きい単位に変換する
	 * @param number $bytes
	 * @return number|string
	 */
	public static function prettyByte2Str($bytes=0)
	{
		if ($bytes >= 1073741824) {
			$bytes = number_format($bytes / 1073741824) . 'GB';
		} elseif ($bytes >= 1048576) {
			$bytes = number_format($bytes / 1048576) . 'MB';
		} elseif ($bytes >= 1024) {
			$bytes = number_format($bytes / 1024) . 'KB';
		} elseif ($bytes > 1) {
			$bytes = $bytes . ' bytes';
		} elseif ($bytes == 1) {
			$bytes = $bytes . ' byte';
		} else {
			$bytes = '0 bytes';
		}
		return $bytes;
	}
}