<?php
/**
 * [Core]Validation：バリデーションクラス
 * 入力検証を行う
 *
 * @link http://komaphp.k-akimoto-matsu.net/Reference#subtitle0 リファレンス@Core.Validation
 * @author TakanoriMatsushige < http://k-akimoto-matsu.net/Inquiry >
 * @copyright k_akimoto_matsu. All Rights Reserved
 */
class Validation{
	/**
	 * 入力フォーム設定に基づきバリデーションを行う
	 * @param unknown $formName fwcore/Config/inputのフォーム名
	 * @return NULL|unknown
	 */
	public static function validate($formName= null){
		if($formName == "" || $formName == null){
			return [];
		}

		$inputconfig = Config::get("input",$formName);

		$res = [];
		foreach($inputconfig as $k=>$v){

			//hidden,labelはチェックしない
			if($v["type"]=="hidden" || $v["type"]=="label") continue;

			if(Helper::ak("validate", $v)){

				$msg = null;

				$val = Input::get($v["name"]);

				//表示なしの場合はチェックしない
				if(Helper::ak("nodisp",$v)){
					$nod = explode("=",$v["nodisp"]);
					if(!is_null(Input::get($nod[0])) && Input::get($nod[0]) == $nod[1]){
						continue;
					}
				}
				$valSet = explode("|",$v["validate"]);
				foreach($valSet as $vs){
					$chk = true;
					$jv = explode("-",$vs);
					switch($jv[0]){
						//必須　スペースは許可しない
						case "requierd":
							if(!is_array($val)){
								if(trim(mb_convert_kana($val, "s", 'UTF-8')) == ""){ //全角スペース・半角スペースのみも未入力とみなす
									$chk = false;
								}
							}else{
								if(count($val) == 0){
									$chk = false;
								//それ以外はファイルとみなす
								}elseif(Helper::ak("tmp_name",$val) && $val["tmp_name"] === ""){ //ファイルサイズが大きすぎて空になる場合もあるので注意
									$chk = false;
								}
							}
							break;
						case "max":
							if(trim($val) != "" && mb_strlen($val) > $jv[1]){
								$chk = false;
							}
							break;
						case "min":
							if(trim($val) != "" && mb_strlen($val) < $jv[1]){
								$chk = false;
							}
							break;
							//数値的に最大値

						case "nummax":
							if(trim($val) != "" && ctype_digit(str_replace(".","",$val)) && $val > $jv[1]){
								$chk = false;
							}
							break;
							//数値的に最小値
						case "nummin":
							if(trim($val) != "" && ctype_digit(str_replace(".","",$val)) && $val < $jv[1]){
								$chk = false;
							}
							break;
							//画像サイズ的最大値
						case "sizemax":
							if(isset($val["size"]) && $val["size"] > $jv[1]){
								$chk = false;
							}
							break;
							//画像サイズ的最小値
						case "sizemin":
							if(isset($val["size"]) && $val["size"] < $jv[1]){
								$chk = false;
							}
							break;
							//半角英数のみ
						case "hankaku":
							if(trim($val) != "" && !preg_match("/^[a-zA-Z0-9._-]+$/", $val)){
								$chk = false;
							}
							break;
						case "number":
							//0,1はtrue,falseのため.""を付加して文字列化する
							if(trim($val) != "" && !ctype_digit($val."")){
								$chk = false;
							}
							break;
							//対象入力コントロールの入力内容と一致するか
						case "match":
							if(trim($val) != "" && !is_null(Input::get($jv[1])) && trim(Input::get($jv[1])) != "" && $val != Input::get($jv[1]) ){
								$chk = false;
							}
							break;
						case "mail":
							if(trim($val) != ""){
								if(!filter_var($val, FILTER_VALIDATE_EMAIL)){
									$chk = false;
								}elseif(mb_strlen($val) > Config::get("app","mail_count")){
									$chk = false;
								}
							}
							break;
						case "url":
							if(trim($val) != ""){
								if(!filter_var($val, FILTER_VALIDATE_URL)){
									$chk = false;
								}
							}
							break;
						case "img":
							if($val["tmp_name"] !== ""){
								if(!is_uploaded_file($val["tmp_name"])){
									$chk = false;
								}elseif(!getimagesize($val["tmp_name"])){
									$chk = false;
								}else{
									$exts = explode(",",$jv[1]);
									$ext = substr($val["name"], strrpos($val["name"], '.') + 1);
									$t = false;
									foreach($exts as $vv){
										if($ext === $vv){
											$t = true;
										}
										if($ext === strtoupper($vv)){
											$t = true;
										}
									}

									//getimagesize関数で画像情報を取得する
									list($img_width, $img_height, $mime_type, $attr) = getimagesize($val["tmp_name"]);
									switch($mime_type){
										//許可をする拡張子
										case IMAGETYPE_JPEG:
										case IMAGETYPE_PNG:
										case IMAGETYPE_GIF:
										case IMAGETYPE_WBMP:
											break;
										default:
											$t = false;

									}

									if(!$t){
										$chk = false;
									}
								}
							}
							break;
						case "file":
							if($val["tmp_name"] !== ""){
								if(!is_uploaded_file($val["tmp_name"])){
									$chk = false;
								}else{
									$exts = explode(",",$jv[1]);
									$ext = substr($val["name"], strrpos($val["name"], '.') + 1);
									$t = false;
									foreach($exts as $vv){
										if($ext === $vv){
											$t = true;
										}
										if($ext === strtoupper($vv)){
											$t = true;
										}
									}

									if(!$t){
										$chk = false;
									}
								}
							}
							break;
							//規定のものか（規定のもの以外が選ばれてたらNG）
						case "type":
							if($jv[1] != ""){
								$types = explode(",",$jv[1]);
								$t = false;
								foreach($types as $vv){
									if($val === $vv){
										$t = true;
									}
								}
								if(!$t){
									$chk = false;
								}
							}
							break;
						case "zip":
							if(trim($val) == ""){
								$chk = false;
								break;
							}
							$d = explode("-",$val);
							if(count($d) != 2){
								$chk = false;
								break;
							}
							if(!is_numeric($d[0])){
								$chk = false;
								break;
							}
							if(strlen($d[0]) != 3){
								$chk = false;
								break;
							}
							if(!is_numeric($d[1])){
								$chk = false;
								break;
							}
							if(strlen($d[1]) != 4){
								$chk = false;
								break;
							}
							break;
						//指定日付以前は×
						case "dateout":
							if(trim($val) != "" && strtotime($val) < strtotime($jv[1])){
								$chk = false;
							}
							break;
						//日付型チェック
						case "date":
							if(trim($val) != "" && $val !== date("Y/m/d",strtotime($val))){
								$chk = false;
							}
							break;
						//日付時間型チェック
						case "datetime":
							$val1 = Input::get($v["name"]."Date");
							if(Helper::ak("requierd",$v) && $v["requierd"] && is_null($val1) || trim($val1) == ""){
								$chk = false;
							}
							if(trim($val1) != "" && $val1 !== date("Y/m/d",strtotime($val1))){
								$chk = false;
							}
							$val2 = Input::get($v["name"]."Time");
							if(trim($val2) != "" && $val2 !== date("H:i",strtotime($val2))){
								$chk = false;
							}
							break;

						//日付時間秒型チェック
						case "datetimeseconds":
							$val1 = Input::get($v["name"]."Date");
							if(trim($val1) != "" && $val1 !== date("Y/m/d",strtotime($val1))){
								$chk = false;
							}
							$val2 = Input::get($v["name"]."Time");
							if(trim($val2) != "" && $val2 !== date("H:i:s",strtotime($val2))){
								$chk = false;
							}
							break;
						//時分型チェック
						case "hitime":
							$val = Input::get($v["name"]);
							if(trim($val) != "" && $val !== date("H:i",strtotime($val))){
								$chk = false;
							}
							break;
						//時分秒型チェック
						case "hitimeseconds":
							$val = Input::get($v["name"]);
							if(trim($val) != "" && $val !== date("H:i:s",strtotime($val))){
								$chk = false;
							}
							break;
						//期間日付型、逆転チェック
						case "dateft":
							$fromname = $v["name"]."From";
							$toname = $v["name"]."To";

							$val1 = Input::get($fromname);
							$val2 = Input::get($toname);

							if(trim($val1) != "" && $val1 !== date("Y/m/d",strtotime($val1))){
								$chk = false;
							}
							if(trim($val2) != "" && $val2 !== date("Y/m/d",strtotime($val2))){
								$chk = false;
							}
							if($chk && trim($val1) != "" && trim($val2) != ""){
								if(strtotime($val1) > strtotime($val2)){
									$chk = false;
								}
							}
							break;
						//時間型、逆転チェック
						case "hift":
							$fromname = $v["name"]."From";
							$toname = $v["name"]."To";
							$val1 = Input::get($fromname);
							$val2 = Input::get($toname);

							if(trim($val1) != "" && $val1 !== date("H:i",strtotime($val1))){
								$chk = false;
							}
							if(trim($val2) != "" && $val2 !== date("H:i",strtotime($val2))){
								$chk = false;
							}
							if($chk && trim($val1) != "" && trim($val2) != ""){
								if(strtotime($val1) > strtotime($val2)){
									$chk = false;
								}
							}
							break;
							//時分型比較チェック
						case "hitimecompare":
							$val1 = Input::get($v["name"]);
							$val2 = Input::get($jv[1]);
							if(!is_null($val1) && trim($val1) != "" && $val1 === date("H:i",strtotime($val1)) && !is_null($val2) && trim($val2) != "" && $val2 === date("H:i",strtotime($val2))){
								if(strtotime(date("Y/m/d ").$val1) < strtotime(date("Y/m/d ").$val2)){
									$chk = false;
								}
							}
							break;
							//日付型比較チェック
						case "datecompare":
							$val1date = Input::get($v["name"]);
							$val2date = Input::get($jv[1]);
								//日付チェック
							//全て入力がある場合のみチェックを行う
							if(trim($val1date) != "" && trim($val2date) != "" ){
								//日付型が合っていることが前提なので日付NGの場合はこのチェックを通さない
								if($val1date !== date("Y/m/d",strtotime($val1date)) || $val2date !== date("Y/m/d",strtotime($val2date))
										){
								}else{
									$v1 = strtotime($val1date);
									$v2 = strtotime($val2date);
									if($v1 > $v2){
										$chk = false;
									}
								}
							}

							break;
						//日付時間型チェック
						case "datetimecompare":
							$val1date = Input::get($v["name"]."Date");
							$val1time = Input::get($v["name"]."Time");
							$val2date = Input::get($jv[1]."Date");
							$val2time = Input::get($jv[1]."Time");
							//日付チェック
							//全て入力がある場合
							if(trim($val1date) != "" && trim($val2time) != "" && trim($val2date) != "" && trim($val2time) != ""){
								//日付形式がおかしい場合はdatetimeチェックでひっかかるのでスルー
								if($val1date !== date("Y/m/d",strtotime($val1date)) || $val1time !== date("H:i",strtotime($val1time)) ||
									$val2date !== date("Y/m/d",strtotime($val2date)) || $val2time !== date("H:i",strtotime($val2time))
									){
								}else{
									$v1 = strtotime($val1date."".$val1time);
									$v2 = strtotime($val2date."".$val2time);
									if($v1 > $v2){
										$chk = false;
									}else{
										$v1 = strtotime($val1date."".$val1time);
										$v2 = strtotime($val1date."".$val2time);

										if($v1 > $v2){
											$chk = false;
										}
									}
								}
							//時間なし
							}elseif(trim($val1date) != "" && trim($val2time) == "" && trim($val2date) != "" && trim($val2time) == ""){
								//日付形式がおかしい場合はdatetimeチェックでひっかかるのでスルー
								if($val1date !== date("Y/m/d",strtotime($val1date)) ||
										$val2date !== date("Y/m/d",strtotime($val2date))
										){
								}else{
									$v1 = strtotime($val1date);
									$v2 = strtotime($val2date);
									if($v1 > $v2){
										$chk = false;
									}
								}
							//2の日付なし
							}elseif(trim($val1date) != "" && trim($val2time) != "" && trim($val2date) == "" && trim($val2time) != ""){
								//日付形式がおかしい場合はdatetimeチェックでひっかかるのでスルー
								if($val1date !== date("Y/m/d",strtotime($val1date)) || $val1time !== date("H:i",strtotime($val1time)) ||
										$val2time !== date("H:i",strtotime($val2time))
										){
								}else{
									$v1 = strtotime($val1date."".$val1time);
									$v2 = strtotime($val1date."".$val2time);
									if($v1 > $v2){
										$chk = false;
									}
								}
							}

							break;
						//DB重複チェック
						case "overlap":
							if(trim($val) != ""){
								$overlap = explode("@",$jv[1]);

								$modelName = $overlap[0];
								$where[$v["name"]] = $val;
								if(isset($overlap[1])){
									$overlap_columns = explode(",",$overlap[1]);
									foreach($overlap_columns as $column){
										if(trim(Input::get($column)) != ""){
											$where[$column] = Input::get($column);
										}
									}
								}
								$mo = DbModel::read($modelName);
								if(!is_null($mo)){
									$re = $mo->getOneInfo($where);
									if(count($re) > 0){
										$chk = false;
									}
								}
							}
							break;
						//パスワード
						case "password":

							if(trim($val) == ""){
								$chk = false;
							}elseif(!preg_match("/^[!-\/:-@\(\)\[\]\{\}~%&=|\^a-zA-Z0-9_-]+$/", $val)){
								$chk = false;
							}elseif(mb_strlen($val) < Config::get("app","password_cnt")){
								$chk = false;
							}else{

								$ct = 0;
								//英数記号が含まれているかチェック
								if(preg_match("/[!-\/:_@\(\)\[\]\{\}~%&=|\^]/", $val)){
									$ct++;
								}
								if(preg_match("/[a-z]/", $val)){
									$ct++;
								}
								if(preg_match("/[A-Z]/", $val)){
									$ct++;
								}
								if(preg_match("/[0-9]/", $val)){
									$ct++;
								}
								if( $ct < 3){
									$chk = false;
								}
							}
							break;
					}
					if(!$chk){
						$res[$v["name"]][] = Config::get("message",$jv[0]);
					}
				}
			}
		}
		return $res;

	}

	/**
	 * 単発バリデータ
	 * @param unknown $val ・・・対象の値
	 * @param unknown $setting　・・・設定方法 Config/input のvalidateと同じ
	 * @param unknown $prefix　・・・メッセージにフィールド名等冒頭句をつける場合は指定
	 * @return NULL|unknown
	 */
	public static function single_validate($val=null,$setting=null,$prefix=null){
		if($setting == "" || $setting == null){
			return [];
		}
		$res = [];

		$msg = null;

		$valSet = explode("|",$setting);
		foreach($valSet as $vs){
			$chk = true;
			$jv = explode("-",$vs);
			switch($jv[0]){
				//必須　スペースは許可しない
				case "requierd":
					if(!is_array($val)){
						if(trim($val) == ""){
							$chk = false;
						}
					}else{
						if(count($val) == 0){
							$chk = false;
							//それ以外はファイルとみなす
						}elseif($val["tmp_name"] === ""){ //ファイルサイズが大きすぎて空になる場合もあるので注意
							$chk = false;
						}
					}
					break;
				case "max":
					if(trim($val) != "" && mb_strlen($val) > $jv[1]){
						$chk = false;
					}
					break;
				case "min":
					if(trim($val) != "" && mb_strlen($val) < $jv[1]){
						$chk = false;
					}
					break;
					//数値的に最大値
				case "nummax":
					if(trim($val) != "" && ctype_digit(str_replace(".","",$val."")) && $val > $jv[1]){
						$chk = false;
					}
					break;
					//数値的に最小値
				case "nummin":
					if(trim($val) != "" && ctype_digit(str_replace(".","",$val."")) && $val < $jv[1]){
						$chk = false;
					}
					break;
					//画像サイズ的最大値
				case "sizemax":
					if(isset($val["size"]) && $val["size"] > $jv[1]){
						$chk = false;
					}
					break;
					//画像サイズ的最小値
				case "sizemin":
					if(isset($val["size"]) && $val["size"] < $jv[1]){
						$chk = false;
					}
					break;
					//半角英数のみ
				case "hankaku":
					if(trim($val) != "" && !preg_match("/^[a-zA-Z0-9]+$/", $val)){
						$chk = false;
					}
					break;
				case "number":
					//0,1はtrue,falseのため.""を付加して文字列化する
					if(trim($val) != "" && !ctype_digit($val."")){
						$chk = false;
					}
					break;
					//対象入力コントロールの入力内容と一致するか
				case "match":
					if(trim($val) != "" && !is_null(Input::get($jv[1])) && trim(Input::get($jv[1])) != "" && $val != Input::get($jv[1]) ){
						$chk = false;
					}
					break;
					//ファイル
				case "file":
					if($val["tmp_name"] !== ""){
						if(!is_uploaded_file($val["tmp_name"])){
							$chk = false;
						}else{
							$exts = explode(",",$jv[1]);
							$ext = substr($val["name"], strrpos($val["name"], '.') + 1);
							$t = false;
							foreach($exts as $vv){
								if($ext === $vv){
									$t = true;
								}
								if($ext === strtoupper($vv)){
									$t = true;
								}
							}

							if(!$t){
								$chk = false;
							}
						}
					}
					break;
				case "img":
					if($val["tmp_name"] !== ""){
						if(!is_uploaded_file($val["tmp_name"])){
							$chk = false;
						}elseif(!getimagesize($val["tmp_name"])){
							$chk = false;
						}else{
							$exts = explode(",",$jv[1]);
							$ext = substr($val["name"], strrpos($val["name"], '.') + 1);
							$t = false;
							foreach($exts as $vv){
								if($ext === $vv){
									$t = true;
								}
								if($ext === strtoupper($vv)){
									$t = true;
								}
							}

							if(!$t){
								$chk = false;
							}
						}
					}
					//規定のものか（規定のもの以外が選ばれてたらNG）
				case "type":
					if($jv[1] != ""){
						$types = explode(",",$jv[1]);
						$t = false;
						foreach($types as $vv){
							if($val === $vv){
								$t = true;
							}
						}
						if(!$t){
							$chk = false;
						}
					}
					break;
					break;
				case "zip":
					if(trim($val) == ""){
						$chk = false;
						break;
					}
					$d = explode("-",$val);
					if(count($d) != 2){
						$chk = false;
						break;
					}
					if(!is_numeric($d[0])){
						$chk = false;
						break;
					}
					if(strlen($d[0]) != 3){
						$chk = false;
						break;
					}
					if(!is_numeric($d[1])){
						$chk = false;
						break;
					}
					if(strlen($d[1]) != 4){
						$chk = false;
						break;
					}
					break;
					//指定日付以前は×
				case "dateout":
					if(trim($val) != "" && strtotime($val) < strtotime($jv[1])){
						$chk = false;
					}
					break;
					//指定日付以降は×
				case "datein":
					if(trim($val) != "" && strtotime($val) > strtotime($jv[1])){
						$chk = false;
					}
					break;
					//日付型チェック
				case "date":
					if(trim($val) != "" && $val !== date("Y/m/d",strtotime($val))){
						$chk = false;
					}
					break;
					//日付時間型チェック
				case "datetime":
					$dtValues = explode(" ",$val);
					$val1 = $dtValues[0];
					if(trim($val1) != "" && $val1 !== date("Y/m/d",strtotime($val1))){
						$chk = false;
					}
					$val2 = $dtValues[1];
					if(trim($val2) != "" && $val2 !== date("H:i",strtotime($val2))){
						$chk = false;
					}
					break;
					//日付時間型チェック
				case "datetimeseconds":
					$dtValues = explode(" ",$val);
					$val1 = $dtValues[0];
					if(trim($val1) != "" && $val1 !== date("Y/m/d",strtotime($val1))){
						$chk = false;
					}
					$val2 = $dtValues[1];
					if(trim($val2) != "" && $val2 !== date("H:i:s",strtotime($val2))){
						$chk = false;
					}
					break;
					//時分型チェック
				case "hitime":
					if(trim($val) != "" && $val !== date("H:i",strtotime($val))){
						$chk = false;
					}
					break;
					//時分秒型チェック
				case "hitimeseconds":
					if(trim($val) != "" && $val !== date("H:i:s",strtotime($val))){
						$chk = false;
					}
					break;
					//パスワード
				case "password":
					if(trim($val) == ""){
						$chk = false;
					}elseif(!preg_match("/^[!-\/:-@\(\)\[\]\{\}~%&=|\^a-zA-Z0-9_-]+$/", $val)){
						$chk = false;
					}elseif(mb_strlen($val) < Config::get("app","password_cnt")){
						$chk = false;
					}else{

						$ct = 0;
						//英数記号が含まれているかチェック
						if(preg_match("/[!-\/:_@\(\)\[\]\{\}~%&=|\^]/", $val)){
							$ct++;
						}
						if(preg_match("/[a-z]/", $val)){
							$ct++;
						}
						if(preg_match("/[A-Z]/", $val)){
							$ct++;
						}
						if(preg_match("/[0-9]/", $val)){
							$ct++;
						}
						if( $ct < 3){
							$chk = false;
						}
					}
					break;
					//ワンタイムトークンの一致チェック
				case "token":
					$chk = Session::checkToken($val);
					break;
			}

			if(!$chk){
				$res[] = $prefix.Config::get("message",$jv[0]);
			}
		}

		//exit;
		return $res;
	}
}