<?php
/**
 * [Model]Model：モデル基底クラス
 * モデル親クラス（子モデル経由以外で直接利用されることはほとんどない）
 *
 * @link http://komaphp.k-akimoto-matsu.net/Reference#subtitle1 リファレンス@Model
 * @author TakanoriMatsushige < http://k-akimoto-matsu.net/Inquiry >
 * @copyright k_akimoto_matsu. All Rights Reserved
 */
class Model{
	public $table = null;
	public $key = null;
	public $properties = [];

	public function __set($name,$value =null){
		$this->properties[$name] = $value;
	}
	public function __get($name){
		return Html::hsp($this->properties[$name]);
	}

	/**
	 * データを挿入する前の成型
	 * @param unknown $inputName　入力画面名
	 * @param array $addD　何か追加するデータがあればkey=>value　※同名があれば上書きしてしまうので注意
	 * @return boolean
	 */
	public function insert($inputName = null,$addD=[]){
		return $this->exeDb($inputName,[],$addD,"i");
	}

	/**
	 * データを更新する前の成型
	 * @param unknown $inputName　入力画面名
	 * @param unknown $where　更新先データ
	 * @param array $addD　何か追加するデータがあればkey=>value　※同名があれば上書きしてしまうので注意
	 * @return count
	 */
	public function update($inputName = null,$where=[],$addD=[]){
		return $this->exeDb($inputName,$where,$addD,"u");
	}

	/**
	 * データを削除する前の成型
	 * @param unknown $inputName　入力画面名
	 * @param unknown $where　更新先データ
	 * @return count
	 */
	public function delete($inputName = null,$where=[]){
		return $this->exeDb($inputName,$where,[],"d");
	}

	/**
	 * DBアクセス（Core/DbModel実行）
	 * @param unknown $inputName
	 * @param array $where
	 * @param array $addD
	 * @param string $act
	 * @return NULL|boolean
	 */
	private function exeDb($inputName = null,$where=[],$addD=[],$act= "i"){
		if(is_null($this->table) || $this->table == "" || is_null($inputName) || $inputName == "" || is_null(Config::get("input",$inputName))) return null;
		$data=[];

		$inputA = Config::get("input",$inputName);
		foreach($inputA as $k=>$v){
			//表示なしの場合はチェックしない
			if(Helper::ak("nodisp",$v)){
				$nod = explode("=",$v["nodisp"]);
				if(!is_null(Input::get($nod[0])) && Input::get($nod[0]) == $nod[1]){
					continue;
				}
			}

			switch($v["type"]){
				case "label":
					break;
					//パターン増えたら定義も増やす
				case "select":
					if(trim(Input::get($v["name"])) == ""){
						$data[$v["name"]] =	null;
					}
					break;
				case "date":
					if(trim(Input::get($v["name"])) == ""){
						$data[$v["name"]] =	null;
					}else{
						$data[$v["name"]] = date("Y-m-d",strtotime(Input::get($v["name"])));
					}
					break;
				case "datetime":
					if(trim(Input::get($v["name"]."Date")) == ""){
						$data[$v["name"]] =	null;
						if(Helper::ak("nullinit",$v)){
							$data[$v["name"]] =	Input::get($v["nullinit"]."Date");
						}
					}else{
						$data[$v["name"]] = date("Y-m-d",strtotime(Input::get($v["name"]."Date")));
					}
					if(trim(Input::get($v["name"]."Time")) == ""){
						$data[$v["time_name"]] =	null;
					}else{
						$data[$v["time_name"]] = str_replace(":","",Input::get($v["name"]."Time"));
					}
					break;
				case "dateft":
					if(trim(Input::get($v["name"])) == ""){
						$data[$v["name"]] =	null;
					}else{
						$data[$v["name"]] = date("Y-m-d",strtotime(Input::get($v["name"])));
					}
					if(trim(Input::get("to_".$v["name"])) == ""){
						$data["to_".$v["name"]] =	null;
					}else{
						$data["to_".$v["name"]] = date("Y-m-d",strtotime(Input::get("to_".$v["name"])));
					}
					break;
				case "hift":
					if(trim(Input::get($v["name"])) == ""){
						$data[$v["name"]] =	null;
					}else{
						$data[$v["name"]] = date("Hi",strtotime(Input::get($v["name"])));
					}
					if(trim(Input::get("to_".$v["name"])) == ""){
						$data["to_".$v["name"]] =	null;
					}else{
						$data["to_".$v["name"]] = date("Hi",strtotime(Input::get("to_".$v["name"])));
					}
					break;
				case "check":
					$ss=[];
					if(is_null(Input::get($v["name"]))){
						$ss[] = 0;
					}else{
						$d = Input::get($v["name"]);
						foreach($d as $vs){
							$ss[] =$vs;
						}
						if(count($ss) == 0){
							$ss[] = 0;
						}
					}
					$data[$v["name"]] = join("|",$ss);
					break;
				case "file":
					//パターン増えたら定義も増やす
					$file = Input::get($v["name"]);
					if(!is_null($file) && $file["name"] !== ""){

						if(!is_uploaded_file($file["tmp_name"])){
							return false;
						}
						$ext = substr($file["name"], strrpos($file["name"], '.') + 1);
						if(Helper::ak($v["name"],$addD)){
							$upFileName = $addD[$v["name"]];
						}else{
							$upFileName = $file["name"];
						}

						if(Helper::ak("upname",$v)){
							if(!is_null(Input::get($v["upname"]))){
								$upName = Input::get($v["upname"]);
							}elseif(Helper::ak($v["upname"],$where)){
								$upName = $where[$v["upname"]];
							}elseif(Helper::ak($v["upname"],$addD)){
								$upName = $addD[$v["upname"]];
							}else{
								$upName = $v["upname"];
							}
							$upFileName = $upName.".".$ext;
						}else{
							$upFileName = $file["name"];
						}
						//保存する内容
						//0・・・ファイル名 1・・・拡張子
						if(Helper::ak("upvalue",$v) && $v["upvalue"] == 1){
							$data[$v["name"]] = max($f);
						}else{
							$data[$v["name"]] = Helper::es($file["name"]);
						}

						//アップ先親ディレクトリ
						$pdir = null;
						if($k == "image"){
							$pdir = "img";
						//
						//その他はtmpへ
						}else{
							$pdir = "tmp";
						}

						$updir = null;
						if(Helper::ak("updir",$v)){
							$updir = $v["updir"];
							$cdir = __DIR__.DS."..".DS."storage".DS.$pdir.DS.$updir;
							if(!file_exists($cdir)){
								mkdir($cdir);
							}
							$pdir .= DS.$updir;
						}else{
						}

						$upf = __DIR__.DS."..".DS."storage".DS.$pdir.DS.$upFileName;

						//既にある場合は削除する
						if(file_exists($upf)){
							unlink($upf);
						}

						//ファイルアップロードの失敗を拾う
						if(!move_uploaded_file($file["tmp_name"], $upf)){
							return false;
						}

					}
					break;
				case "password":
					//パスワード系は入力あるときのみ更新
					if(trim(Input::get($v["name"])) != ""){
						$data[$v["name"]] = hash(Config::get("app","hash_key"),Config::get("database","prefix")."_".Input::get($v["name"]) );
					}
					break;
				default:
					$data[$v["name"]] = Input::get($v["name"]);
			}

			//子もある場合
			//連動する子要素がある場合
			if(Helper::ak("child",$v) && $v["child"]){
				//空要素を置くのみ
				$subvalue = null;
				switch($v["subtype"]){
					case "label":
						break;
						//パターン増えたら定義も増やす
					case "select":
						if(trim(Input::get($v["subname"])) == ""){
							$data[$v["subname"]] =	null;
						}else{
							$data[$v["subname"]] =	Input::get($v["subname"]);
						}
						break;
					default:
						$data[$v["subname"]] = Input::get($v["subname"]);
				}
			}
		}

		//登録日時日・更新日時
		$data["update_date"] = date("Y-m-d H:i:s");
		if($act == "i"){ //登録日時はiのみ
			$data["create_date"] = date("Y-m-d H:i:s");
		}
		//固定値など
		if(count($addD) > 0){
			foreach($addD as $k=>$v){
				$data[$k] = $v;
			}
		}
		$inputT = Config::get("tables",Config::get("database","prefix")."_".$this->table); //テーブルにあるカラムのみ取り出す
		$dataa = [];
		foreach($data as $clumn=>$v){
			if(Helper::ak($clumn,$inputT)){
				$dataa[$clumn] = $v;
				Log::set("{$clumn}=>{$v}");
			}
		}

		if(count($dataa) == 0) return false; //ひとつも該当がない場合はNG
		if($act == "u"){
			return DbModel::update(Config::get("database","prefix")."_".$this->table,$dataa,$where);
		}elseif($act == "i"){
			return DbModel::insert(Config::get("database","prefix")."_".$this->table,$dataa);
		}elseif($act == "d"){
			return DbModel::delete(Config::get("database","prefix")."_".$this->table,$where);
		}
	}

	/**
	 * 指定IDから1件だけデータを取得する
	 * @param unknown $t
	 * @param unknown $id
	 * @return unknown[]
	 */
	public function getByID($t=null,$id = null){
		if(is_null($id)) return null;
		$w = [
			$t=>$id
		];
		return DbModel::select(Config::get("database","prefix")."_".$this->table,$w,true);
	}

	/**
	 * 汎用データ取得
	 * @param array $where
	 * @param string $t
	 * @param array $order
	 * @return unknown[]
	 */
	protected function get($where=[],$t=false,$order=[]){
		return DbModel::select(Config::get("database","prefix")."_".$this->table,$where,$t,$order);
	}

	/**
	 * 検索設定から検索結果データを取得する
	 * @param unknown $listSetting
	 * @param array $order
	 * @return unknown[]
	 */
	public function getBySearch($listSetting=null,$order =[]){

		$where = $this->createWhere($listSetting);

		$o = null;
		if(count($order) > 0){
			$o = " order by  ";
			$cnt = 0;
			foreach($order as $k=>$v){
				if($cnt > 0){
					$o .=" , ";
				}
				$by = null;
				switch($v){
					case 0:
						$by = " asc ";
						break;
					case 1:
						$by = " desc ";
						break;

				}
				$o .= " {$k} {$by} ";
				$cnt++;
			}
		}
		$sql = "select * from ".Config::get("database","prefix")."_".$this->table." {$where} {$o}";
		$ret = DbModel::execute($sql);
		$datas = [];
		$cnt=0;
		while($row = DbModel::fetch($ret)){
			$datas[$cnt] = $row;
			$cnt++;
		}
		return $datas;
	}

	/**
	 * 検索設定からWhere部分を作る
	 * @param unknown $listSetting
	 * @return NULL|string
	 */
	protected function createWhere($listSetting){
		if(is_null($listSetting) || $listSetting == "" || is_null(Config::get("search",$listSetting))) return null;
		$where = null;
		$searchA = Config::get("search",$listSetting);

		if(!is_null(Session::get($listSetting))){
			$search = Session::get($listSetting);
		}else{
			$search = Input::all();
		}

		$where = null;
		$cnt = 0;
		foreach($searchA as $k=>$v){
			if(Helper::ak($v["name"],$search) && !is_null($search[$v["name"]]) && trim(mb_convert_kana($search[$v["name"]], "s", 'UTF-8')) != ""){
				if($cnt > 0){
					$where .=" and ";
				}

				$clm = $this->replaceClmName($v["name"]);
				switch($v["type"]){
					case "check":
						$dval = [];
						foreach($search[$v["name"]] as $val){
							$dval[] = DbModel::es($val);
						}
						$where .= " {$clm} in (".join(",",$dval).") ";
						break;
					case "date":
						$db = Config::get("database","db");
						switch($db){
							case "mysql":
								$where .= " date_format({$clm},'%Y/%m/%d') like '".DbModel::es($search[$v["name"]],false)."%' ";
								break;
							case "postgres":
								$where .= " to_char({$clm},'YYYY/MM/DD') like '".DbModel::es($search[$v["name"]],false)."%' ";
								break;
						}
						break;
					default:
						if(Helper::ak("stype",$v)){
							switch($v["stype"]){
								case 0:
									$where .= " {$clm} = ".DbModel::es($search[$v["name"]])." ";
									break;
								case 1:
									$where .= " {$clm} like '%".DbModel::es($search[$v["name"]],false)."%' ";
									break;
							}
						}
						break;
				}
				$cnt++;
			}
		}

		if(!is_null($where)){
			$where = " where ".$where;
		}

		return $where;
	}

	/**
	 * 数値キーの最大値＋１を取得する
	 * @return unknown
	 */
	public function createId(){
		$sql = "select max(".$this->key.") + 1 as nextid from ".Config::get("database","prefix")."_".$this->table;
		Log::set($sql);
		$ret = DbModel::execute($sql);
		$data = DbModel::fetch($ret);
		return $data["nextid"];
	}

	//__を.に変更する（joinしたテーブル対応）
	protected function replaceClmName($name){
		return str_replace("__",".",$name);
	}
}